package com.yfd.platform.modules.algorithm.service.impl;

import com.yfd.platform.modules.algorithm.domain.AlgorithmArrangeDevice;
import com.yfd.platform.modules.algorithm.mapper.AlgorithmArrangeDeviceMapper;
import com.yfd.platform.modules.algorithm.service.IAlgorithmArrangeDeviceService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author zhengsl
 * @since 2025-06-06
 */
@Service
public class AlgorithmArrangeDeviceServiceImpl extends ServiceImpl<AlgorithmArrangeDeviceMapper,
        AlgorithmArrangeDevice> implements IAlgorithmArrangeDeviceService {

    @Resource
    private AlgorithmArrangeDeviceMapper algorithmArrangeDeviceMapper;

    /***********************************
     * 用途说明: 查询当前布点的主设备数量
     * 参数说明 arrangeId 算法布点ID
     * 返回值说明: java.util.List<java.util.Map<java.lang.String,java.lang.Object>>
     ***********************************/
    @Override
    public List<Map<String, Object>> getArrangeDeviceInfo(String arrangeId) {
        List<Map<String, Object>> rawData = algorithmArrangeDeviceMapper.getArrangeDeviceInfo(arrangeId);

        // 使用 Map<主设备ID, 结果行> 来合并数据
        Map<String, Map<String, Object>> groupedData = new HashMap<>();

        for (Map<String, Object> row : rawData) {
            String areaId = (String) row.get("areaId");
            String areaName = (String) row.get("areaName");
            String deviceTypeCode = (String) row.get("deviceTypeCode");
            Long count = (Long) row.get("deviceTypeCount");

            groupedData.computeIfAbsent(areaId, k -> {
                Map<String, Object> newRow = new HashMap<>();
                newRow.put("areaId", areaId);
                newRow.put("areaName", areaName);
                return newRow;
            });
            // 设置具体类型的数量
            groupedData.get(areaId).put(deviceTypeCode, count);
        }

        return new ArrayList<>(groupedData.values());
    }
}
