package com.yfd.platform.modules.algorithm.service.impl;

import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.yfd.platform.config.HttpServerConfig;
import com.yfd.platform.modules.algorithm.domain.AlgorithmArrange;
import com.yfd.platform.modules.algorithm.domain.AlgorithmArrangeDevice;
import com.yfd.platform.modules.algorithm.mapper.AlgorithmArrangeMapper;
import com.yfd.platform.modules.algorithm.service.IAlgorithmArrangeDeviceService;
import com.yfd.platform.modules.algorithm.service.IAlgorithmArrangeService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yfd.platform.modules.basedata.domain.SubstationMaindevice;
import com.yfd.platform.modules.basedata.service.ISubstationMaindeviceService;
import com.yfd.platform.utils.FileUtil;
import com.yfd.platform.utils.HttpRESTfulUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author zhengsl
 * @since 2025-06-06
 */
@Service
@Slf4j
public class AlgorithmArrangeServiceImpl extends ServiceImpl<AlgorithmArrangeMapper, AlgorithmArrange> implements IAlgorithmArrangeService {

    @Resource
    private HttpRESTfulUtils httpRESTfulUtils;

    @Resource
    private ISubstationMaindeviceService substationMaindeviceService;
    @Resource
    private IAlgorithmArrangeDeviceService algorithmArrangeDeviceService;
    @Resource
    private HttpServerConfig httpServerConfig;

    /***********************************
     * 用途说明: 调用算法布点API
     * 参数说明 id : 算法布点ID
     * 返回值说明: com.yfd.platform.modules.algorithm.domain.AlgorithmArrange
     ***********************************/
    @Override
    public AlgorithmArrange callAlgorithmArrange(String id) {
        // 查询算法布点信息
        AlgorithmArrange algorithmArrange = this.getById(id);
        if (algorithmArrange == null) {
            return null;
        }
        if ("0".equals(algorithmArrange.getIsImageFlag())) {
            return algorithmArrange;
        }
        JSONObject param = new JSONObject();
        param.put("arrange_id", algorithmArrange.getId());
        param.put("station_name", algorithmArrange.getStationName());
        param.put("station_type", algorithmArrange.getStationType());
        param.put("volt_level", algorithmArrange.getVoltLevel());
        param.put("business_type", algorithmArrange.getBusinessType());
        String deviceTypeList = algorithmArrange.getDeviceTypeList();
        List<String> typeList = StrUtil.split(deviceTypeList, ",");
        LambdaQueryWrapper<SubstationMaindevice> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.in(SubstationMaindevice::getDeviceType, typeList).select(SubstationMaindevice::getMainDeviceId,
                SubstationMaindevice::getMainDeviceName, SubstationMaindevice::getDeviceType,
                SubstationMaindevice::getAreaId, SubstationMaindevice::getAreaName);
        List<Map<String, Object>> maps = substationMaindeviceService.listMaps(queryWrapper);
        List<Map<String, Object>> mainDeviceList = renameKeysWithStream(maps);
        param.put("main_device_list", mainDeviceList);
        File file = new File("D:\\riis\\video\\test0.jpg");
        JSONObject arrange = httpRESTfulUtils.uploadJsonWithImageFileWithDataWrapper("192.168.1.173", "20014",
                "arrange", param, file
                , null);
        if (!"200".equals(arrange.getString("code"))) {
            throw new RuntimeException("调用失败！");
        }
        String arrangeId = arrange.getString("arrange_id");
        JSONArray resultsList = arrange.getJSONArray("results_list");
        List<AlgorithmArrangeDevice> algorithmArrangeDeviceList = new java.util.ArrayList<>();
        for (int i = 0; i < resultsList.size(); i++) {
            JSONObject result = resultsList.getJSONObject(i);
            String areaId = result.getString("area_id");
            String areaName = result.getString("area_name");
            JSONArray patroldeviceList = result.getJSONArray("patroldevice_list");
            for (int j = 0; j < patroldeviceList.size(); j++) {
                AlgorithmArrangeDevice algorithmArrangeDevice = new AlgorithmArrangeDevice();
                JSONObject patroldevice = patroldeviceList.getJSONObject(j);
                String patroldeviceType = patroldevice.getString("patroldevice_type");
                String patroldevicePos = patroldevice.getString("patroldevice_pos");
                algorithmArrangeDevice.setArrangeId(arrangeId);
                algorithmArrangeDevice.setAreaId(areaId);
                algorithmArrangeDevice.setAreaName(areaName);
                algorithmArrangeDevice.setPatroldeviceType(patroldeviceType);
                algorithmArrangeDevice.setPatroldevicePos(patroldevicePos);
                algorithmArrangeDeviceList.add(algorithmArrangeDevice);
            }
        }
        algorithmArrangeDeviceService.saveBatch(algorithmArrangeDeviceList);
        return algorithmArrange;

    }

    /***********************************
     * 用途说明: 上传文件
     * 参数说明 file 文件域
     * 返回值说明: java.lang.String
     ***********************************/
    @Override
    public String uploadImage(MultipartFile file) {
        String extensionName = FileUtil.getExtensionName(file.getOriginalFilename());
        if ("blob".equals(extensionName)) {
            extensionName = "png";
        }
        // 文件存储地址
        String fileName =
                IdUtil.fastSimpleUUID() + "." + extensionName;
        // 上传文件
        String name =
                Objects.requireNonNull(FileUtil.upload(file, httpServerConfig.getPlanFilePath(), fileName)).getName();
        return name;
    }

    @Override
    public boolean removeImage(String filePath) {
        Path path = Paths.get(filePath);
        try {
            if (Files.exists(path)) {
                Files.deleteIfExists(path);
                log.info("文件删除成功: {}", filePath);
                return true;
            } else {
                log.warn("文件不存在，跳过删除: {}", filePath);
                return false;
            }
        } catch (IOException e) {
            log.error("删除文件 {} 时发生异常", filePath, e);
            return false;
        }
    }

    /***********************************
     * 用途说明: 转换方法
     * 参数说明 originalList 原始列表
     * 返回值说明: java.util.List<java.util.Map<java.lang.String,java.lang.Object>>
     ***********************************/
    private List<Map<String, Object>> renameKeysWithStream(List<Map<String, Object>> originalList) {
        return originalList.stream().map(originalMap -> {
            Map<String, Object> newMap = new HashMap<>();
            newMap.put("main_device_id", originalMap.get("mainDeviceId"));
            newMap.put("main_device_name", originalMap.get("mainDeviceName"));
            newMap.put("device_type", originalMap.get("deviceType"));
            newMap.put("area_id", originalMap.get("areaId"));
            newMap.put("area_name", originalMap.get("areaName"));
            return newMap;
        }).collect(Collectors.toList());
    }
}
